unit UnidadProcedimientos;

interface
  uses inifiles, sysutils, ShlObj, ActiveX, Windows, ShellCtrls,
      controls, classes, registry, db, dialogs, forms, variants;

  function leBoolINI (clave, cadena : string; defecto : boolean) : boolean;
  function leCadINI (clave, cadena : string; defecto : String) : String;
  function leNumINI (clave, cadena : string; defecto : Integer) : Integer;
  procedure esCadINI (clave, cadena : string; valor : String);
  procedure esBoolINI (clave, cadena : string; valor : Boolean);
  procedure esNumINI (clave, cadena : string; valor : Integer);
  function SeleccionarCarpeta (titulo : string) : string;
  procedure cargarConfiguracion ();
  function encriptar(aStr: String; aKey: Integer): String;
  function desencriptar(aStr: String; aKey: Integer): String;
  function obtenerCadenaRegistro (claveInicial : Cardinal; clave : string; valor : string) : string;
  procedure obtenerDatosODBC (odbc : string; sistema : boolean;
      var bd : string; var tipo : string; var descripcion : string;
      var controlador : string; var servidor : string;
      var listener : string; var usuario : string);
  procedure conectarBD ();
  function comprobarDuplicado (nombre : string; tabla : string; campo : string) : boolean;
  function insertarvalor (valor : string; tabla : string) : integer;
  function obtenerValor (campo : string; tabla : string; valor : Integer) : string;
  procedure actualizarStock (entrada : boolean; cantidad : double; codigo : Integer);
  function obtenerNombre (codigo : integer; tabla : string) : string;
  function generarCodigoAuto (tabla : string) : integer;
var
  vtUsuario : string; //usuario para acceso a la BD (si no se ha marcado Pedir Usuario
  vtContrasena : string;  //contrasea para acceso a la BD (si no se ha marcado Pedir Usuario
  vtTipoBDMySQL : boolean;
  vtDefectoTipo : String; //guarda el tipo de tercero por defecto (proveedor, cliente, otro)
  vtDefectoCambioControl : Boolean; //permitir cambiar de foco con la tecla Intro
  vtDuplicadoNombreArticulo : Boolean; //permitir duplicados del nombre del artculo
  vtDuplicadoNombreTercero : Boolean; //permitir duplicados del nombre del tercero
  vtBDMySQLNombreBD : string; //nombre de la base de datos
  vtBDMySQLServidor : string; //nombre/ip del servidor donde est alojada la BD
  vtBDMySQLPuerto : string;  //puerto utilizado para conexin a MySQL
  vtBDMySQLProtocolo : string; //versin de MySQL que se utilizar, admite Oracle
  vtBDMySQLPedirUsuario : boolean; //mostrar ventana de peticin de usuario y contrasea para acceso a la BD

  vtClaveElegida : integer; //aqu se almacenar el cdigo del registro seleccionado en la ventana de seleccin

  vtAccion : string; //para la ventana de modificacin, insercin, visualizacin (acciones)
  vtCodigo: integer; //almacena el cdigo del registro a visualizar
  vtCodigoTexto : string; //almacena el cdigo del registro a visualizar (texto)  
  vtIndeterminado: string; //almacena el valor por defecto para un campo indeterminado

  vtCrearFacturaAuto : boolean; //crea la factura automticamente mostrando la ventana de insercin de artculos
                              //como cdigo de cliente por defecto utiliza vtCodigoClienteFacturas
  vtCodigoClienteFacturas : integer; //cdigo de cliente por defecto en facturas
  vtCodigoProveedorFacturas : integer; //cdigo de proveedor por defecto en facturas

  vtEmpresaNombre : string;
  vtEmpresaDireccion : string;
  vtEmpresaCIF : string;

  //para seleccionar tercero
  vtCodigoTerceroFactura : integer;


const
  //nombre de las tablas de la aplicacin
  vtTablaTercero = 'tercero';
  vtTablaArticulo = 'articulo';
  vtTablaFactura = 'factura';
  vtTablaCobro = 'cobro';
  vtTablaTalla = 'talla';
  vtTablaColor = 'color';
  vtTablaMarca = 'marca';
  vtTablaModelo = 'modelo';
  vtTablaGenero = 'genero';
  vtTablaRango = 'rango';
  vtTablaFamilia = 'familia';
  vtTablaFacturaDetalle = 'facturadetalle';
  vtTablaFacturaProveedorDetalle = 'facturaproveedordetalle';
  vtTablaFacturaProveedor = 'facturaproveedor';
  vtTablaPago = 'pago';
  vtTablaGasto = 'gasto';
  vtTablaContador = 'contador';
  vtTablaSQL = 'sql';
  vtTablaVale = 'vale';

  //tag para ventana de seleccin
  vtNumProveedor = 10;
  vtNumCliente = 20;
  vtNumArticulo = 30;
  vtNumGenero = 40;
  vtNumMarca = 50;
  vtNumTalla = 60;
  vtNumRango = 70;
  vtNumModelo = 80;
  vtNumFamilia = 90;
  vtNumColor = 100;
  vtNumFacturaLinea = 110;
  vtNumFactura = 120;
  vtNumGasto = 130;
  vtNumCobro = 140;
  vtNumPago = 150;
  vtNumVale = 160;

implementation

uses UnidadModuloDatos, UnidadPeticionUsuario, UnidadMenuprincipal,
  ZConnection, ZAbstractRODataset;

  
//Genera el cdigo de la tabla especificada automticamente
function generarCodigoAuto (tabla : string) : integer;
var
  contador : integer;
begin
  with md.tcContador do
  begin
    Close;
    SQL.Clear;  //obtenemos el contador de la tabla contadores
    SQL.Add('SELECT *');
    SQL.Add('FROM ' + vtTablaContador);
    SQL.Add('WHERE tabla = :pTabla');
    ParamByName('pTabla').DataType := ftString;
    ParamByName('pTabla').Value := tabla;
    Open;
    if not varisnull (md.tcContador.FieldByName('Contador').value) then
    begin
      try
        contador := FieldByName('contador').Value + 1;
        //Incrementamos el contador de la tabla contadores
        Close;
        SQL.Clear;
        SQL.Add('UPDATE ' + vtTablaContador +
            ' SET contador = :pContador + 1 WHERE tabla = :pTabla');
        ParamByName('pContador').DataType := ftInteger;
        ParamByName('pContador').Value := contador;
        ParamByName('pTabla').DataType := ftString;
        ParamByName('pTabla').Value := tabla;
        ExecSQL;
        close;
        result := contador;
      except
        Close;
        result := 1;
        MessageDlg ('El cdigo no ha podido generarse automticamente.', mtError, [mbOK], 0);
      end;
    end
    else  //si no existe el registro de contador para la tabla actual, lo creamos
    begin
      try
        Close;
        SQL.Clear;
        SQL.Add('INSERT INTO ' + vtTablaContador +
            ' (tabla, contador) ' +
            ' VALUES (:pTabla, 1)');
        ParamByName('pTabla').DataType := ftString;
        ParamByName('pTabla').Value := tabla;
        ExecSQL;
        result := 1;
        Close;
      except
        Close;
        MessageDlg ('El contador no se ha guardado correctamente.', mtError, [mbOK], 0);
      end;
    end;
  end;
end;


function obtenerNombre (codigo : integer; tabla : string) : string;
begin
  md.tc2.Close;
  md.tc2.SQL.Clear;
  md.tc2.SQL.Add('SELECT Codigo, Nombre ' + ' FROM ' + tabla +
      ' WHERE codigo = :pCodigo');
  md.TC2.ParamByName('pCodigo').Value := codigo;
  md.TC2.Open;
  if md.TC2.RecordCount > 0 then
    result := md.TC2.fieldbyname('Nombre').value
  else
    result := '';
end;


//Actualiza el stock de los artculos (facturas clientes - salidas / facturas proveedores - entradas
procedure actualizarStock (entrada : boolean; cantidad : double; codigo : Integer);
begin
  with md.tcStock do
  begin
    close;
    SQL.Clear;
    SQL.Add('UPDATE ' + vtTablaArticulo);
    if entrada then
      SQL.Add('SET stock = stock + :pCantidad')
    else
      SQL.Add('SET stock = stock - :pCantidad');
    SQL.Add('WHERE Codigo = :pCodigo');
    ParamByName('pCantidad').DataType := ftFloat;
    ParamByName('pCantidad').Value := cantidad;
    ParamByName('pCodigo').DataType := ftInteger;
    ParamByName('pCodigo').Value := codigo;
    execsql;
    close;
  end;
end;


//inserta un registro en la tabla especificada con el campo nombre=valor
//devuelve el cdigo del registro insertado
function insertarvalor (valor : string; tabla : string) : integer;
begin
  with md.tcIndeterminado do
  begin
    Close;
    SQL.Clear;
    SQL.Add('SELECT count(codigo) cuenta FROM ' + tabla + ' WHERE nombre = ' + QuotedStr(AnsiUpperCase(valor)));
    Open;
    if fieldbyname('cuenta').asinteger = 0 then
    begin
      close;
      SQL.Clear;
      SQL.Add('INSERT INTO ' + tabla + ' (nombre) VALUES (' + QuotedStr(valor) + ')');
      ExecSQL;
      close;
    end;
    close;
    SQL.Clear;
    SQL.Add('SELECT codigo FROM ' + tabla + ' WHERE nombre = ' + QuotedStr(AnsiUpperCase(valor)));
    open;
    result := fieldbyname('codigo').value;
    close;
  end;
end;

//obtiene el valor del campo y la tabla especificados
function obtenerValor (campo : string; tabla : string; valor : Integer) : string;
begin
  Result := '';
  with md.tcIndeterminado do
  begin
    Close;
    SQL.Clear;
    SQL.Add('SELECT ' + campo + ' FROM ' + tabla + ' WHERE codigo = ' + inttostr(valor));
    Open;
    Result := fieldbyname(campo).AsString;
    close;
  end;
end;

function comprobarDuplicado (nombre : string; tabla : string; campo : string) : boolean;
begin
  with md.tc do
  begin
    Close;
    SQL.Clear;
    SQL.Add('SELECT COUNT(' + campo + ') Contador');
    SQL.Add('FROM ' + tabla);
    SQL.Add('WHERE UPPER(' + campo + ') = :pNombre');
    ParamByName('pNombre').DataType := ftString;
    ParamByName('pNombre').Value := AnsiUpperCase(nombre);
    Open;
    result := FieldByName('Contador').value > 0;
    close;
  end;
end;


function SeleccionarCarpeta (titulo : string) : string;
var
  Inf: TBrowseInfo;
  Identificador: PItemIDList;
  Gestor : IMalloc;
  Carpeta : Array[0..MAX_PATH] Of Char;
 // handle : THandle;
begin
  With Inf Do
  Begin
    //hwndOwner := Handle;
    pidlRoot := Nil;
    pszDisplayName := Carpeta;
    lpszTitle := pchar(titulo);
    ulFlags := BIF_BROWSEINCLUDEFILES;
    lpfn := Nil;
  End;
  Identificador := SHBrowseForFolder(Inf);
  If Identificador <> Nil Then
  Begin
    SHGetPathFromIDList(Identificador, Carpeta);
    SHGetMalloc(Gestor);
    Gestor.Free(Identificador);
    SeleccionarCarpeta := Carpeta
  end;
end;


//lee un valor booleano de un fichero INI
function leBoolINI (clave, cadena : string; defecto : boolean) : boolean;
begin
  with tinifile.create (changefileext(paramstr(0),'.INI')) do
  try
    result := readbool (clave, cadena, defecto);
  finally
    free;
  end;
end;

//lee un valor numrico de un fichero INI
function leNumINI (clave, cadena : string; defecto : Integer) : Integer;
begin
  with tinifile.create (changefileext(paramstr(0),'.INI')) do
  try
    result := ReadInteger (clave, cadena, defecto);
  finally
    free;
  end;
end;

//lee un valor string de un fichero INI
function leCadINI (clave, cadena : string; defecto : String) : String;
begin
  with tinifile.create (changefileext(paramstr(0),'.INI')) do
  try
    result := ReadString (clave, cadena, defecto);
  finally
    free;
  end;
end;

//escribe un valor string en un fichero INI
procedure esCadINI (clave, cadena : string; valor : String);
begin
  with tinifile.create (changefileext(paramstr(0),'.INI')) do
  try
    WriteString (clave, cadena, valor);
  finally
    free;
  end;
end;

//escribe un valor boolean en un fichero INI
procedure esBoolINI (clave, cadena : string; valor : Boolean);
begin
  with tinifile.create (changefileext(paramstr(0),'.INI')) do
  try
    WriteBool (clave, cadena, valor);
  finally
    free;
  end;
end;

//escribe un valor numrico en un fichero INI
procedure esNumINI (clave, cadena : string; valor : Integer);
begin
  with tinifile.create (changefileext(paramstr(0),'.INI')) do
  try
    WriteInteger (clave, cadena, valor);
  finally
    free;
  end;
end;


//encriptar datos
function encriptar(aStr: String; aKey: Integer): String;
begin
   Result:='';
   RandSeed:=aKey;
   for aKey:=1 to Length(aStr) do
       Result:=Result+Chr(Byte(aStr[aKey]) xor random(256));
end;

//desencriptar datos
function desencriptar(aStr: String; aKey: Integer): String;
begin
   Result := '';
   RandSeed := aKey;
   for aKey := 1 to Length(aStr) do
       Result := Result + Chr(Byte(aStr[aKey]) xor random(256));
end;


procedure cargarConfiguracion ();
begin
  vtTipoBDMySQL := leBoolINI('Base de datos', 'Tipo BD MySQL', false);
  vtBDMySQLNombreBD := leCadINI('Base de datos', 'Tipo BD MySQL - Nombre BD', '');
  vtBDMySQLServidor := leCadINI('Base de datos', 'Tipo BD MySQL - Servidor', '');
  vtBDMySQLPedirUsuario := leBoolINI('Base de datos', 'Tipo BD MySQL - Pedir usuario al inicio', false);
  vtBDMySQLProtocolo := leCadINI('Base de datos', 'Tipo BD MySQL - Protocolo', 'mysql-4.1');
  vtBDMySQLPuerto := leCadINI('Base de datos', 'Tipo BD MySQL - Puerto', '3306');
  //vtContrasena := desencriptar (leCadINI('Base de datos', 'Tipo BD MySQL - Contrasea', ''), 450);
  vtContrasena := leCadINI('Base de datos', 'Tipo BD MySQL - Contrasea', '');
  vtUsuario := leCadINI('Base de datos', 'Tipo BD MySQL - Usuario', '');
  vtDefectoTipo := leCadINI('Defecto', 'Tipo tercero', 'Cliente');
  vtDefectoCambioControl := leBoolINI('Defecto', 'Cambio control intro', true);
  vtDuplicadoNombreArticulo := leBoolINI('Defecto', 'Permitir duplicado - Nombre articulo', true);
  vtDuplicadoNombreTercero := leBoolINI('Defecto', 'Permitir duplicado - Nombre tercero', true);     
  vtClaveElegida := -1;
  vtTipoBDMySQL := true;
  vtIndeterminado := leCadINI('Defecto', 'Valor indeterminado', 'Indeterminado');
  vtEmpresaNombre := leCadINI('Empresa', 'Nombre', '');
  vtEmpresaDireccion := leCadINI('Empresa', 'Direccin', '');
  vtEmpresaCIF := leCadINI('Empresa', 'CIF', '');
  vtCodigoClienteFacturas := leNumINI('Defecto', 'Cdigo cliente defecto facturas', 1);
  vtCodigoProveedorFacturas := leNumINI('Defecto', 'Cdigo proveedor defecto facturas', 1);  
  vtCrearFacturaAuto := leBoolINI('Defecto', 'Crear factura automticamente', true);
end;

function obtenerCadenaRegistro (claveInicial : Cardinal; clave : string; valor : string) : string;
var
  reg: TRegistry;
begin
  Result := '';
  Reg := TRegistry.Create;
  Reg.RootKey := claveInicial;
  reg.OpenKey(clave, False);
  try
    Result := reg.ReadString(valor);
  finally
    reg.CloseKey;
  end;
end;

procedure obtenerDatosODBC (odbc : string; sistema : boolean;
    var bd : string; var tipo : string; var descripcion : string;
    var controlador : string; var servidor : string;
    var listener : string; var usuario : string);
var
  claveInicial : Cardinal;
begin
  bd := '';
  tipo := '';
  descripcion := '';
  controlador := '';
  servidor := '';
  listener := '';
  usuario := '';
  if sistema then
    claveInicial := HKEY_LOCAL_MACHINE
  else
    claveInicial := HKEY_CURRENT_USER;
  tipo := obtenerCadenaRegistro(claveInicial,'SOFTWARE\ODBC\ODBC.INI\ODBC Data Sources',odbc);

  //Oracle
  if pos ('ORACLE', AnsiUpperCase (tipo)) > 0 then
  begin
    descripcion := obtenerCadenaRegistro(claveInicial, 'SOFTWARE\ODBC\ODBC.INI\' + odbc, 'Description');
    controlador := obtenerCadenaRegistro(claveInicial, 'SOFTWARE\ODBC\ODBC.INI\' + odbc, 'Driver');
    listener := obtenerCadenaRegistro(claveInicial, 'SOFTWARE\ODBC\ODBC.INI\' + odbc, 'ServerName');
    usuario := obtenerCadenaRegistro(claveInicial, 'SOFTWARE\ODBC\ODBC.INI\' + odbc, 'UserID');
  end;
  //MySQL
  if pos ('MYSQL', AnsiUpperCase (tipo)) > 0 then
  begin
    descripcion := obtenerCadenaRegistro(claveInicial, 'SOFTWARE\ODBC\ODBC.INI\' + odbc, 'DESCRIPTION');
    bd := obtenerCadenaRegistro(claveInicial, 'SOFTWARE\ODBC\ODBC.INI\' + odbc, 'DATABASE');
    controlador := obtenerCadenaRegistro(claveInicial, 'SOFTWARE\ODBC\ODBC.INI\' + odbc, 'Driver');
    servidor := obtenerCadenaRegistro(claveInicial, 'SOFTWARE\ODBC\ODBC.INI\' + odbc, 'SERVER');
    usuario := obtenerCadenaRegistro(claveInicial, 'SOFTWARE\ODBC\ODBC.INI\' + odbc, 'UID');
  end;
  //MS Access
  if pos ('ACCESS', AnsiUpperCase (tipo)) > 0 then
  begin
    descripcion := obtenerCadenaRegistro(claveInicial, 'SOFTWARE\ODBC\ODBC.INI\' + odbc, 'Description');
    bd := obtenerCadenaRegistro(claveInicial, 'SOFTWARE\ODBC\ODBC.INI\' + odbc, 'DBQ');
    controlador := obtenerCadenaRegistro(claveInicial, 'SOFTWARE\ODBC\ODBC.INI\' + odbc, 'Driver');
    usuario := obtenerCadenaRegistro(claveInicial, 'SOFTWARE\ODBC\ODBC.INI\' + odbc, 'UID');
  end;
  //DBASE
  if pos ('DBASE', AnsiUpperCase (tipo)) > 0 then
  begin
    descripcion := obtenerCadenaRegistro(claveInicial, 'SOFTWARE\ODBC\ODBC.INI\' + odbc, 'Description');
    controlador := obtenerCadenaRegistro(claveInicial, 'SOFTWARE\ODBC\ODBC.INI\' + odbc, 'Driver');
  end;
  //SQL SERVER
  if pos ('SQL SERVER', AnsiUpperCase (tipo)) > 0 then
  begin
    descripcion := obtenerCadenaRegistro(claveInicial, 'SOFTWARE\ODBC\ODBC.INI\' + odbc, 'Description');
    controlador := obtenerCadenaRegistro(claveInicial, 'SOFTWARE\ODBC\ODBC.INI\' + odbc, 'Driver');
    servidor := obtenerCadenaRegistro(claveInicial, 'SOFTWARE\ODBC\ODBC.INI\' + odbc, 'Server');
    usuario := obtenerCadenaRegistro(claveInicial, 'SOFTWARE\ODBC\ODBC.INI\' + odbc, 'LastUser');
  end;
  //Paradox
  if pos ('PARADOX', AnsiUpperCase (tipo)) > 0 then
  begin
    descripcion := obtenerCadenaRegistro(claveInicial, 'SOFTWARE\ODBC\ODBC.INI\' + odbc, 'Description');
    controlador := obtenerCadenaRegistro(claveInicial, 'SOFTWARE\ODBC\ODBC.INI\' + odbc, 'Driver');
    bd := obtenerCadenaRegistro(claveInicial, 'SOFTWARE\ODBC\ODBC.INI\' + odbc, 'DefaultDir');
    servidor := obtenerCadenaRegistro(claveInicial, 'SOFTWARE\ODBC\ODBC.INI\' + odbc + '\Engines\Paradox', 'ParadoxNetPath');
    usuario := obtenerCadenaRegistro(claveInicial, 'SOFTWARE\ODBC\ODBC.INI\' + odbc, 'LastUser');
  end;
  //Excel
  if pos ('EXCEL', AnsiUpperCase (tipo)) > 0 then
  begin
    descripcion := obtenerCadenaRegistro(claveInicial, 'SOFTWARE\ODBC\ODBC.INI\' + odbc, 'Description');
    controlador := obtenerCadenaRegistro(claveInicial, 'SOFTWARE\ODBC\ODBC.INI\' + odbc, 'Driver');
    bd := obtenerCadenaRegistro(claveInicial, 'SOFTWARE\ODBC\ODBC.INI\' + odbc, 'DBQ');
    servidor := obtenerCadenaRegistro(claveInicial, 'SOFTWARE\ODBC\ODBC.INI\' + odbc, 'DefaultDir');
  end;
  //TEXT, CSV
  if pos ('TEXT', AnsiUpperCase (tipo)) > 0 then
  begin
    descripcion := obtenerCadenaRegistro(claveInicial, 'SOFTWARE\ODBC\ODBC.INI\' + odbc, 'Description');
    controlador := obtenerCadenaRegistro(claveInicial, 'SOFTWARE\ODBC\ODBC.INI\' + odbc, 'Driver');
    bd := obtenerCadenaRegistro(claveInicial, 'SOFTWARE\ODBC\ODBC.INI\' + odbc, 'DefaultDir');
  end;
end;

                   
procedure conectarBD ();
var
  f : tform;
begin
  if vtTipoBDMySQL then
  begin
    if (vtUsuario = '') or (vtBDMySQLServidor = '') or (vtBDMySQLProtocolo = '') then
      MessageDlg('Debe configurar el tipo de base de datos al que se conectar.',
          mtWarning, [mbok], 0)
    else
    begin
      with md.bd do
      begin
        Connected := false;
        LoginPrompt := false;
        if vtBDMySQLPedirUsuario then
        begin
          F := application.findcomponent('formIncioSesion') as TForm;
          if Assigned(F) then
              f.showmodal
          else
          begin
            application.createform(tformIncioSesion, formIncioSesion);
            if vtTipoBDMySQL then
            begin
              formIncioSesion.txtBD.Text := vtBDMySQLNombreBD;
              formIncioSesion.txtTipoBD.Text := 'MySQL';
            end;
            formIncioSesion.showmodal;
          end;
        end;
        md.bd.Protocol := vtBDMySQLProtocolo;
        HostName := vtBDMySQLServidor;
        Port := strtoint(vtBDMySQLPuerto);
        Database := vtBDMySQLNombreBD;
        User := vtUsuario;
        Password := vtContrasena;
        try
          Connected := true;
        except
          //EZSQLException
          on e : exception do
          begin
            MessageDlg ('No se ha podido conectar con el servidor: [' + vtBDMySQLServidor +
                ']. Compruebe que el servicio de MySQL est iniciado y que ha ' +
                'escrito correctamente el nombre del servidor.' + chr(13) + chr(13) +
                'A continuacin la descripcin completa del error:' + chr(13) +
                e.ClassName + chr(13) + e.Message,
                mtError, [mbok], 0);
            formmenuprincipal.be.Panels[1].Text := 'No conectado';
            Exit;
          end;
        end;
      end;
    end;
  end;
  formmenuprincipal.be.Panels[1].Text := vtBDMySQLProtocolo + '@' +
      vtBDMySQLServidor + '@' + vtUsuario;
end;



end.
